unit DBSpeedButton;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	Buttons, dbtables, db, dbctrls;

type
	TDBSpeedButton = class(TSpeedButton)
	private
		{ Private-Deklarationen }
		FDataLink: TFieldDataLink;
		function GetDataField: string;
		function GetDataSource: TDataSource;
		procedure UpdateData(Sender: TObject);
		procedure DataChange(Sender: TObject);
		procedure EditingChange(Sender: TObject);
		procedure SetDataField(const Value: string);
		procedure SetDataSource(Value: TDataSource);
		procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
	protected
		{ Protected-Deklarationen }
	public
		{ Public-Deklarationen }
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure Click; override;
	published
		{ Published-Deklarationen }
		property DataField: string read GetDataField write SetDataField;
		property DataSource: TDataSource read GetDataSource write SetDataSource;
	end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('LS', [TDBSpeedButton]);
end;

{ TDBSpeedButton }

procedure TDBSpeedButton.Click;
begin
	if (FDataLink.CanModify) then begin
		FDataLink.UpdateRecord;
		inherited Click;
	end;
end;

procedure TDBSpeedButton.CMGetDataLink(var Message: TMessage);
begin
	Message.Result := Integer(FDataLink);
end;

constructor TDBSpeedButton.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	ControlStyle := ControlStyle + [csReplicatable];
	FDataLink := TFieldDataLink.Create;
	FDataLink.Control := Self;
	FDataLink.OnDataChange := DataChange;
	FDataLink.OnEditingChange := EditingChange;
	FDataLink.OnUpdateData := UpdateData;
end;

procedure TDBSpeedButton.DataChange(Sender: TObject);
begin
	if FDataLink.Field <> nil then Down:=FDataLink.Field.AsBoolean;
end;

destructor TDBSpeedButton.Destroy;
begin
	FDataLink.Free;
	FDataLink := nil;
	inherited Destroy;
end;

procedure TDBSpeedButton.EditingChange(Sender: TObject);
begin
end;

function TDBSpeedButton.GetDataField: string;
begin
	Result := FDataLink.FieldName;
end;

function TDBSpeedButton.GetDataSource: TDataSource;
begin
	Result := FDataLink.DataSource;
end;

procedure TDBSpeedButton.MouseDown(Button: TMouseButton;
	Shift: TShiftState; X, Y: Integer);
begin
	if (FDataLink.CanModify) then begin
		inherited MouseDown(Button, Shift, X, Y);
	end;
end;

procedure TDBSpeedButton.MouseUp;
begin
	if (FDataLink.CanModify) then begin
		if not (FDataLink.Editing) then FDataLink.Edit;
		FDataLink.Modified;
		inherited MouseUp(Button, Shift, X, Y);
	end;
end;

procedure TDBSpeedButton.SetDataField(const Value: string);
begin
	FDataLink.FieldName := Value;
end;

procedure TDBSpeedButton.SetDataSource(Value: TDataSource);
begin
	if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
		FDataLink.DataSource := Value;
	if Value <> nil then Value.FreeNotification(Self);
end;

procedure TDBSpeedButton.UpdateData(Sender: TObject);
begin
	FDataLink.Field.AsBoolean:=Down;
end;

end.
